/*
** VF[_vO
*/
#include <iostream>
#include <fstream>

#include "shader.h"

/*
** VF[_[̃\[XvOɓǂݍ
*/
static bool readShaderSource(GLuint shader, const char *name)
{
  bool ret = false; // IXe[^Xifalse ȂG[j

  // \[Xt@CJ
  std::ifstream file(name, std::ios::binary);

  if (file.fail()) {
    // JȂ
    std::cerr << "Error: Can't open source file: " << name << std::endl;
  }
  else {
    // t@C̖Ɉړ݈ʒuit@CTCYj𓾂
    file.seekg(0L, std::ios::end);
    GLsizei length = file.tellg();

    // t@CTCỸm
    char *buffer = new GLchar[length];

    // t@C擪ǂݍ
    file.seekg(0L, std::ios::beg);
    file.read(buffer, length);

    if (file.bad()) {
      // ܂ǂݍ߂Ȃ
      std::cerr << "Error: Could not read souce file: " << name << std::endl;
    }
    else {
      // VF[_̃\[XvÕVF[_IuWFNgւ̓ǂݍ
      const GLchar *source[] = { buffer };
      glShaderSource(shader, 1, source, &length);
      ret = true;
    }
    file.close();

    // ǂݍ݂ɎgJ
    delete[] buffer;
  }

  return ret;
}

/*
** VF[_̏\
*/
static void printShaderInfoLog(GLuint shader)
{
  // VF[_̃RpC̃O̒擾
  GLsizei bufSize;
  glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &bufSize);

  if (bufSize > 1) {
    // VF[_̃RpC̃O̓e擾
    GLchar *infoLog = new GLchar[bufSize];
    GLsizei length;
    glGetShaderInfoLog(shader, bufSize, &length, infoLog);
    std::cerr << "InfoLog:\n" << infoLog << '\n' << std::endl;
    delete[] infoLog;
  }
}

/*
** vȌ\
*/
static void printProgramInfoLog(GLuint program)
{
  // VF[_̃ÑO̒擾
  GLsizei bufSize;
  glGetProgramiv(program, GL_INFO_LOG_LENGTH , &bufSize);

  if (bufSize > 1) {
    // VF[_̃ÑO̓e擾
    GLchar *infoLog = new GLchar[bufSize];
    GLsizei length;
    glGetProgramInfoLog(program, bufSize, &length, infoLog);
    std::cerr << "InfoLog:\n" << infoLog << '\n' << std::endl;
    delete[] infoLog;
  }
}

/*
** VF[_[\[Xt@C̓ǂݍ
*/
GLuint loadShader(const char *vert, const char *frag, const char *geom, GLenum input, GLenum output)
{
  // RpC, N
  GLint compiled, linked;

  // vOIuWFNg̍쐬
  GLuint program = glCreateProgram();

  // o[ebNXVF[_
  GLuint vertShader = glCreateShader(GL_VERTEX_SHADER);
  if (!readShaderSource(vertShader, vert)) return 0;
  glCompileShader(vertShader);
  glGetShaderiv(vertShader, GL_COMPILE_STATUS, &compiled);
  printShaderInfoLog(vertShader);
  if (compiled == GL_FALSE) {
    std::cerr << "Error: Could not compile vertex shader source: " << vert << std::endl;
    return 0;
  }
  glAttachShader(program, vertShader);
  glDeleteShader(vertShader);

  // tOgVF[_
  GLuint fragShader = glCreateShader(GL_FRAGMENT_SHADER);
  if (!readShaderSource(fragShader, frag)) return 0;
  glCompileShader(fragShader);
  glGetShaderiv(fragShader, GL_COMPILE_STATUS, &compiled);
  printShaderInfoLog(fragShader);
  if (compiled == GL_FALSE) {
    std::cerr << "Error: Could not compile fragment shader source: " << frag << std::endl;
    return 0;
  }
  glAttachShader(program, fragShader);
  glDeleteShader(fragShader);

  // WIgVF[_iIvVj
  if (geom) {
    GLuint geomShader = glCreateShader(GL_GEOMETRY_SHADER_EXT);
    if (!readShaderSource(geomShader, geom)) return 0;
    glCompileShader(geomShader);
    glGetShaderiv(geomShader, GL_COMPILE_STATUS, &compiled);
    printShaderInfoLog(geomShader);
    if (compiled == GL_FALSE) {
      std::cerr << "Error: Could not compile geometry shader source: " << geom << std::endl;
      return 0;
    }
    glAttachShader(program, geomShader);
    glDeleteShader(geomShader);

    glProgramParameteriEXT(program, GL_GEOMETRY_INPUT_TYPE_EXT, input);
    glProgramParameteriEXT(program, GL_GEOMETRY_OUTPUT_TYPE_EXT, output);

    int vertices;
    glGetIntegerv(GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT, &vertices);
    glProgramParameteriEXT(program, GL_GEOMETRY_VERTICES_OUT_EXT, vertices);
  }

  // feedback Ɏg varying ϐw肷
  const static char *varyings[] = { "position" };
  glTransformFeedbackVaryings(program, sizeof varyings / sizeof varyings[0], varyings, GL_INTERLEAVED_ATTRIBS);

  // VF[_vÕN
  glLinkProgram(program);
  glGetProgramiv(program, GL_LINK_STATUS, &linked);
  printProgramInfoLog(program);
  if (linked == GL_FALSE) {
    std::cerr << "Error: Could not link shader program" << std::endl;
    return 0;
  }

  return program;
}
